/// <reference types="react" />

import {Extension} from '@codemirror/state'
import {ObjectDefinition} from 'sanity'
import {ObjectInputProps} from 'sanity'
import {ObjectSchemaType} from 'sanity'
import {Plugin as Plugin_2} from 'sanity'
import {PreviewConfig} from 'sanity'
import {PreviewProps} from 'sanity'

/**
 * @public
 */
export declare interface CodeDefinition
  extends Omit<ObjectDefinition, 'type' | 'fields' | 'options'> {
  type: typeof codeTypeName
  options?: CodeOptions
}

/** @public */
export declare function CodeInput(props: CodeInputProps): JSX.Element

/**
 * @public
 */
export declare const codeInput: Plugin_2<void | CodeInputConfig>

declare interface CodeInputConfig {
  codeModes?: CodeMode[]
}

export declare interface CodeInputLanguage {
  title: string
  value: string
  mode?: string
}

/**
 * @public
 */
export declare interface CodeInputProps extends ObjectInputProps<CodeInputValue, CodeSchemaType> {}

/**
 * @public
 */
export declare interface CodeInputValue {
  _type?: 'code'
  code?: string
  filename?: string
  language?: string
  highlightedLines?: number[]
}

declare interface CodeMode {
  name: string
  loader: ModeLoader
}

/**
 * @public
 */
export declare interface CodeOptions {
  theme?: string
  darkTheme?: string
  languageAlternatives?: CodeInputLanguage[]
  language?: string
  withFilename?: boolean
}

/**
 * @public
 */
export declare const codeSchema: {
  type: 'object'
  name: 'code'
} & Omit<ObjectDefinition, 'preview'> & {
    preview?:
      | PreviewConfig<
          {
            language: string
            code: string
            filename: string
            highlightedLines: string
          },
          Record<'code' | 'language' | 'filename' | 'highlightedLines', any>
        >
      | undefined
  }

/**
 * @public
 */
export declare interface CodeSchemaType extends Omit<ObjectSchemaType, 'options'> {
  options?: CodeOptions
}

/**
 * @public
 */
export declare const codeTypeName: 'code'

declare type ModeLoader = () => Promise<Extension | undefined> | Extension | undefined

/**
 * @public
 */
export declare function PreviewCode(props: PreviewCodeProps): JSX.Element

/**
 * @public
 */
export declare interface PreviewCodeProps extends PreviewProps {
  selection?: CodeInputValue
}

export {}

declare module '@sanity/types' {
  interface IntrinsicDefinitions {
    code: CodeDefinition
  }
}
